var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { v4 as uuid } from "uuid";
import { USER_TRAITS_PERSISTENCE_KEY } from "./traits";
import { normalizePhone, parseNumberToString } from "../lib/utils";
const ANONYMOUS_ID_PERSISTENCE_KEY = "journifyio_anonymous_id";
const USER_ID_PERSISTENCE_KEY = "journifyio_user_id";
const EXTERNAL_IDs_PERSISTENCE_KEY = "journifyio_external_ids";
export class UserImpl {
    constructor(stores, sentry, cookiesService, phoneCountryCode) {
        this.traits = {};
        this.stores = stores;
        this.sentry = sentry;
        this.cookieService = cookiesService;
        this.phoneCountryCode = phoneCountryCode;
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.initAnonymousId();
            yield this.initUserId();
            this.initTraits();
            this.initExternalIds();
        });
    }
    identify(userId, traits = {}, externalIds) {
        return __awaiter(this, void 0, void 0, function* () {
            if (userId && userId != this.userId) {
                if (this.userId) {
                    this.clearUserData();
                }
                yield this.setUserId(userId);
                yield this.syncAndSetServerCookies();
            }
            yield this.setTraits(traits);
            if (externalIds) {
                yield this.setExternalIds(externalIds);
            }
        });
    }
    getUserId() {
        return this.userId;
    }
    getAnonymousId() {
        return this.anonymousId;
    }
    getTraits() {
        return this.traits;
    }
    getExternalIds() {
        return this.externalIds;
    }
    syncAndSetServerCookies() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (!this.cookieService)
                    return;
                const httpCookies = yield this.cookieService.dispatchRenew();
                if (httpCookies.anonymousId) {
                    this.anonymousId = httpCookies.anonymousId;
                    this.stores.set(ANONYMOUS_ID_PERSISTENCE_KEY, this.anonymousId);
                }
                if (httpCookies.userId) {
                    this.userId = httpCookies.userId;
                    this.stores.set(USER_ID_PERSISTENCE_KEY, this.userId);
                }
            }
            catch (err) {
                this.sentry.captureException(err);
                console.error(err);
            }
        });
    }
    clearUserData() {
        this.anonymousId = null;
        this.userId = null;
        this.externalIds = null;
        this.traits = null;
        this.stores.remove(ANONYMOUS_ID_PERSISTENCE_KEY);
        this.stores.remove(USER_ID_PERSISTENCE_KEY);
        this.stores.remove(EXTERNAL_IDs_PERSISTENCE_KEY);
        this.stores.remove(USER_TRAITS_PERSISTENCE_KEY);
    }
    initUserId() {
        return __awaiter(this, void 0, void 0, function* () {
            this.userId = this.stores.get(USER_ID_PERSISTENCE_KEY);
            if (this.userId) {
                yield this.setUserId(this.userId);
            }
        });
    }
    initAnonymousId() {
        return __awaiter(this, void 0, void 0, function* () {
            this.anonymousId = this.stores.get(ANONYMOUS_ID_PERSISTENCE_KEY);
            let newlyGenerated = false;
            if (!this.anonymousId) {
                this.anonymousId = uuid();
                newlyGenerated = true;
            }
            this.stores.set(ANONYMOUS_ID_PERSISTENCE_KEY, this.anonymousId);
            if (newlyGenerated) {
                // Only when anonymousId is generated, sync with server cookies
                yield this.syncAndSetServerCookies();
            }
        });
    }
    initExternalIds() {
        return __awaiter(this, void 0, void 0, function* () {
            // get externalIds from one of the stores
            this.externalIds = this.stores.get(EXTERNAL_IDs_PERSISTENCE_KEY);
            if (this.externalIds) {
                // set externalIds on all stores
                yield this.setExternalIds(this.externalIds);
            }
        });
    }
    initTraits() {
        return __awaiter(this, void 0, void 0, function* () {
            const traits = this.stores.get(USER_TRAITS_PERSISTENCE_KEY);
            yield this.setTraits(traits);
        });
    }
    setTraits(newTraits) {
        return __awaiter(this, void 0, void 0, function* () {
            this.traits = Object.assign(Object.assign({}, this.traits), newTraits);
            this.formatPhone();
            this.stores.set(USER_TRAITS_PERSISTENCE_KEY, this.traits);
        });
    }
    formatPhone() {
        var _a, _b;
        if (((_a = this.phoneCountryCode) === null || _a === void 0 ? void 0 : _a.length) > 0 && ((_b = this.traits.phone) === null || _b === void 0 ? void 0 : _b.length) > 0) {
            this.traits.phone = normalizePhone(this.traits.phone, this.phoneCountryCode);
        }
    }
    setUserId(userId) {
        return __awaiter(this, void 0, void 0, function* () {
            this.userId = parseNumberToString(userId);
            this.stores.set(USER_ID_PERSISTENCE_KEY, this.userId);
        });
    }
    setExternalIds(externalIds) {
        return __awaiter(this, void 0, void 0, function* () {
            this.externalIds = externalIds;
            this.stores.set(EXTERNAL_IDs_PERSISTENCE_KEY, externalIds);
        });
    }
}
//# sourceMappingURL=user.js.map