var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { getProductionWriteKey, Loader } from "./api/loader";
import { SentryWrapperImpl } from "./lib/sentry";
import { removeEmptyStrings } from "./lib/utils";
const DEFAULT_CDN_HOST = "https://static.journify.io";
const callsBeforeLoad = [];
const sentryWrapper = new SentryWrapperImpl();
const loader = new Loader(sentryWrapper);
let sdk = null;
let recordingCallsBeforeLoad = false;
function load(sdkSettings) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const wKeySettings = yield fetchWriteKeySettings(sdkSettings);
            sdk = yield loader.load(sdkSettings, wKeySettings);
            callsBeforeLoad.forEach((call) => call());
        }
        catch (error) {
            sentryWrapper.captureException(error);
            console.error(error);
        }
    });
}
function fetchWriteKeySettings(sdkSettings) {
    return __awaiter(this, void 0, void 0, function* () {
        const productionWriteKey = getProductionWriteKey(sdkSettings === null || sdkSettings === void 0 ? void 0 : sdkSettings.writeKey);
        let settings = yield fetchRemoteWriteKeySettings(productionWriteKey, sdkSettings.cdnHost || DEFAULT_CDN_HOST);
        if (!settings) {
            settings = {
                syncs: [],
            };
        }
        return settings;
    });
}
function fetchRemoteWriteKeySettings(writeKey, cdnHost) {
    return __awaiter(this, void 0, void 0, function* () {
        const max_retries = 2;
        const settingsURL = `${cdnHost}/write_keys/${writeKey}.json`;
        for (let i = 0; i < max_retries; i++) {
            try {
                sentryWrapper.setTag("settingsURL", settingsURL);
                const response = yield fetch(settingsURL);
                if (200 <= response.status && response.status <= 299) {
                    const settings = yield response.json();
                    return settings;
                }
                else if (500 <= response.status && response.status <= 599) {
                    if (i < max_retries - 1) {
                        console.log(`Retrying in 2 seconds...`);
                        yield sleep(2000);
                    }
                }
                else if (response.status != 404) {
                    const error = new Error(`write key settings are not found for write key ${writeKey}. Status: ${response.status}`);
                    yield sentryWrapper.setResponse({
                        url: settingsURL,
                        headers: response.headers,
                        status: response.status,
                        body: yield response.text(),
                    });
                    sentryWrapper.captureException(error);
                    console.error(error);
                }
                return { syncs: [] };
            }
            catch (error) {
                sentryWrapper.captureException(error);
                console.error(`Failed to fetch write key settings from ${settingsURL}. error: ${error}`);
            }
        }
        return null;
    });
}
function sleep(ms) {
    return new Promise((resolve) => setTimeout(resolve, ms));
}
function identify(userId, traits, externalIds) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!sdk) {
                recordCallBeforeLoad(() => identify(userId, traits, externalIds));
                return null;
            }
            if (!userId) {
                throw new Error("the user id is required when calling identify");
            }
            traits = removeEmptyStrings(traits);
            loader.startNewSession();
            return sdk.identify(userId, traits, externalIds);
        }
        catch (error) {
            sentryWrapper.captureException(error);
            console.error(error);
        }
    });
}
function track(eventName, properties, traits) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!sdk) {
                recordCallBeforeLoad(() => track(eventName, properties, traits));
                return null;
            }
            traits = removeEmptyStrings(traits);
            return sdk.track(eventName, properties, traits);
        }
        catch (error) {
            sentryWrapper.captureException(error);
            console.error(error);
        }
    });
}
function page(param1, param2, param3) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!sdk) {
                recordCallBeforeLoad(() => page(param1, param2, param3));
                return null;
            }
            let pageName = null;
            let properties = null;
            let traits = null;
            if (param1 && typeof param1 === "string") {
                pageName = param1;
                traits = param3 || {};
            }
            else if (param1 && typeof param1 === "object") {
                properties = param1;
                traits = param2 || {};
            }
            if (param2 && !properties && typeof param2 === "object") {
                properties = param2;
                traits = param3 || {};
            }
            traits = removeEmptyStrings(traits);
            return sdk.page(pageName, properties, traits);
        }
        catch (error) {
            sentryWrapper.captureException(error);
            console.error(error);
        }
    });
}
function group(groupId, traits) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            if (!sdk) {
                recordCallBeforeLoad(() => group(groupId, traits));
                return null;
            }
            if (!groupId) {
                throw new Error("the group id is required when calling group");
            }
            traits = removeEmptyStrings(traits);
            return sdk.group(groupId, traits);
        }
        catch (error) {
            sentryWrapper.captureException(error);
            console.error(error);
        }
    });
}
function recordCallBeforeLoad(call) {
    try {
        while (recordingCallsBeforeLoad)
            ;
        recordingCallsBeforeLoad = true;
        callsBeforeLoad.push(call);
        recordingCallsBeforeLoad = false;
    }
    catch (error) {
        sentryWrapper.captureException(error);
        console.error(error);
    }
}
export { load, identify, track, page, group };
//# sourceMappingURL=index.js.map