import { matchFilters } from "../lib/filters";
import { facebookBirthday, oneLetterGender, toDigitsOnlyPhone, toLowerCase, trim, } from "../lib/tranformations";
import { JournifyEventType } from "../../../domain/event";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
const STANDARD_EVENTS = new Set([
    "AddPaymentInfo",
    "AddToCart",
    "AddToWishlist",
    "CompleteRegistration",
    "Contact",
    "CustomizeProduct",
    "Donate",
    "FindLocation",
    "InitiateCheckout",
    "Lead",
    "Purchase",
    "PageView",
    "Schedule",
    "Search",
    "StartTrial",
    "SubmitApplication",
    "Subscribe",
    "ViewContent",
]);
const USER_DATA_FIELDS = new Set([
    "em",
    "fn",
    "ln",
    "ph",
    "external_id",
    "ge",
    "db",
    "ct",
    "st",
    "zp",
    "country",
]);
const FACEBOOK_SCRIPT_URL = "https://connect.facebook.net/en_US/fbevents.js";
const DEFAULT_CURRENCY = "USD";
export class FacebookPixel {
    constructor(deps) {
        this.name = "facebook";
        this.settings = {};
        this.user = deps.user;
        this.browser = deps.browser;
        this.fieldsMapper = deps.fieldMapperFactory.newFieldMapper(deps.sync.field_mappings);
        this.testingMode = deps.testingWriteKey;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    track(ctx) {
        return this.trackPixelEvent(ctx);
    }
    page(ctx) {
        return this.trackPixelEvent(ctx);
    }
    group(ctx) {
        return this.trackPixelEvent(ctx);
    }
    identify(ctx) {
        const newEvent = ctx.getEvent();
        const storedEvent = getStoredIdentify(this.user);
        const event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initPixel(event);
        return this.trackPixelEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    trackPixelEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event);
        const eventId = mappedProperties.event_id;
        delete mappedProperties.event_id;
        const eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        const trackType = isStandardEvent(eventName)
            ? "trackSingle"
            : "trackSingleCustom";
        const properties = this.removeUserData(mappedProperties);
        if (!properties.currency) {
            properties.currency = DEFAULT_CURRENCY;
        }
        this.callPixelHelper(trackType, this.settings.pixel_id, eventName, properties, { eventID: eventId });
        return ctx;
    }
    removeUserData(properties) {
        const output = {};
        for (const key in properties) {
            if (USER_DATA_FIELDS.has(key)) {
                continue;
            }
            output[key] = properties[key];
        }
        return output;
    }
    init(sync) {
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`Facebook Pixel ${this.settings.pixel_id} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadFbScript();
        }
        const event = getStoredIdentify(this.user);
        this.initPixel(event);
    }
    loadFbScript() {
        const localWindow = this.browser.window();
        if (localWindow.fbq) {
            return;
        }
        let n = null;
        n = localWindow.fbq = function (...args) {
            n.callMethod ? n.callMethod(...args) : n.queue.push(args);
        };
        if (!localWindow._fbq)
            localWindow._fbq = n;
        n.push = n;
        n.loaded = !0;
        n.version = "2.0";
        n.queue = [];
        this.browser.injectScript(FACEBOOK_SCRIPT_URL, { async: true });
    }
    initPixel(identifyEvent) {
        const userData = this.mapUserData(identifyEvent);
        this.callPixelHelper("init", this.settings.pixel_id, userData);
    }
    callPixelHelper(...args) {
        if (this.testingMode) {
            this.logger.log("Will call window.fbq with the following params in order:", args);
            return;
        }
        this.browser.window().fbq(...args);
    }
    mapUserData(event) {
        const transformationsMap = {
            em: [trim, toLowerCase],
            fn: [trim, toLowerCase],
            ln: [trim, toLowerCase],
            ph: [toDigitsOnlyPhone],
            city: [trim, toLowerCase],
            st: [trim, toLowerCase],
            country: [toLowerCase],
            ge: [oneLetterGender, toLowerCase],
            db: [facebookBirthday],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    }
}
function isStandardEvent(eventName) {
    return STANDARD_EVENTS.has(eventName);
}
//# sourceMappingURL=facebookPixel.js.map