import { matchFilters } from "../lib/filters";
import { JournifyEventType } from "../../../domain/event";
import { toLowerCase, trim } from "../lib/tranformations";
import { getStoredIdentify } from "../lib/identify";
import { toSettingsObject } from "../lib/settings";
const SNAPCHAT_PIXEL_SCRIPT_URL = "https://sc-static.net/scevent.min.js";
export class SnapchatPixel {
    constructor(deps) {
        this.name = "snapchat_pixel";
        this.user = deps.user;
        this.browser = deps.browser;
        this.eventMapperFactory = deps.eventMapperFactory;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    identify(ctx) {
        const newEvent = ctx.getEvent();
        const storedEvent = getStoredIdentify(this.user);
        const event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        this.initPixel(event);
        return this.trackPixelEvent(ctx);
    }
    track(ctx) {
        return this.trackPixelEvent(ctx);
    }
    group(ctx) {
        return this.trackPixelEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    page(ctx) {
        return this.trackPixelEvent(ctx);
    }
    trackPixelEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event);
        const eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        const properties = this.transformProperties(mappedProperties);
        this.callPixelHelper("track", eventName, properties);
        return ctx;
    }
    transformProperties(properties) {
        if (properties.item_category) {
            if (properties.item_category instanceof Array) {
                if (properties.item_category.length > 1) {
                    properties.item_category = [properties.item_category.join(",")];
                }
                if (properties.item_category.length === 1) {
                    properties.item_category = properties.item_category[0];
                }
            }
        }
        if (properties.item_ids instanceof Array &&
            properties.item_ids.length > 0) {
            properties.item_ids = [properties.item_ids.join(";")];
        }
        if (properties.number_items instanceof Array &&
            properties.number_items.length > 0) {
            properties.number_items = properties.number_items.join(";");
        }
        if (properties.transaction_id) {
            properties.transaction_id = properties.transaction_id.toString();
        }
        return properties;
    }
    init(sync) {
        this.eventMapper = this.eventMapperFactory.newEventMapper(sync.event_mappings);
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`Snapchat Pixel ${this.settings.pixel_id} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadScript();
        }
        const event = getStoredIdentify(this.user);
        this.initPixel(event);
    }
    initPixel(identifyEvent) {
        const userData = this.mapUserData(identifyEvent);
        this.callPixelHelper("init", this.settings.pixel_id, userData);
    }
    callPixelHelper(trackingType, ...args) {
        if (this.testingMode) {
            this.logger.log("Will call window.snaptr with the following params in order:", [trackingType, ...args]);
        }
        else {
            this.browser.window().snaptr(trackingType, ...args);
        }
    }
    mapUserData(event) {
        const transformationsMap = {
            user_email: [trim, toLowerCase],
            ip_address: [trim, toLowerCase],
            user_hashed_email: [trim, toLowerCase],
            user_hashed_phone_number: [trim, toLowerCase],
            firstname: [trim, toLowerCase],
            lastname: [trim, toLowerCase],
            geo_city: [trim, toLowerCase],
            geo_region: [trim, toLowerCase],
            geo_postal_code: [trim, toLowerCase],
            geo_country: [trim, toLowerCase],
            age: [trim],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    }
    /* eslint-disable */
    loadScript() {
        const localWindow = this.browser.window();
        if (localWindow.snaptr)
            return;
        const a = (localWindow.snaptr = function () {
            a.handleRequest
                ? a.handleRequest.apply(a, arguments)
                : a.queue.push(arguments);
        });
        a.queue = [];
        const s = "script";
        const localDocument = this.browser.document();
        const r = localDocument.createElement(s);
        r.async = !0;
        r.src = SNAPCHAT_PIXEL_SCRIPT_URL;
        const u = localDocument.getElementsByTagName(s)[0];
        u.parentNode.insertBefore(r, u);
    }
}
//# sourceMappingURL=snapchatPixel.js.map