/* eslint-disable  @typescript-eslint/no-explicit-any */
import { JournifyEventType } from "../../../domain/event";
import { matchFilters } from "../lib/filters";
import { EventMapperImpl, } from "../lib/mapping";
import { toLowerCase, trim } from "../lib/tranformations";
import { toSettingsObject } from "../lib/settings";
import { getStoredIdentify } from "../lib/identify";
const TIKTOK_SCRIPT_URL = "https://analytics.tiktok.com/i18n/pixel/events.js";
export class TikTokPixel {
    constructor(deps) {
        this.name = "tiktok_pixel";
        this.user = deps.user;
        this.eventMapper = deps.eventMapperFactory.newEventMapper(deps.sync.event_mappings);
        this.browser = deps.browser;
        this.fieldMapperFactory = deps.fieldMapperFactory;
        this.testingMode = deps.testingWriteKey;
        this.logger = deps.logger;
        this.init(deps.sync);
    }
    identify(ctx) {
        const newEvent = ctx.getEvent();
        const storedEvent = getStoredIdentify(this.user);
        const event = {
            type: JournifyEventType.IDENTIFY,
            userId: newEvent.userId || storedEvent.userId,
            anonymousId: newEvent.anonymousId || storedEvent.anonymousId,
            traits: Object.assign(Object.assign({}, (storedEvent.traits || {})), (newEvent.traits || {})),
        };
        const userData = this.mapUserData(event);
        this.callPixelHelper(event.type, userData);
        return this.trackPixelEvent(ctx);
    }
    track(ctx) {
        return this.trackPixelEvent(ctx);
    }
    page(ctx) {
        return this.trackPixelEvent(ctx);
    }
    group(ctx) {
        return this.trackPixelEvent(ctx);
    }
    updateSettings(sync) {
        this.init(sync);
    }
    init(sync) {
        this.fieldsMapper = this.fieldMapperFactory.newFieldMapper(sync.field_mappings, () => new Date());
        this.eventMapper = new EventMapperImpl(sync.event_mappings);
        this.settings = toSettingsObject(sync.settings);
        if (this.testingMode) {
            this.logger.log(`Tiktok Pixel ${this.settings.pixel_code} is detected, but script is not injected because you are using a testing write key.`);
        }
        else {
            this.loadTiktokPixel();
        }
    }
    mapUserData(event) {
        const transformationsMap = {
            email: [trim, toLowerCase],
            phone_number: [trim],
            external_id: [trim],
        };
        return this.fieldsMapper.mapEvent(event, transformationsMap);
    }
    callPixelHelper(eventType, args, eventId = "") {
        const event = Object.assign(Object.assign({}, args), { pixelCode: this.settings.pixel_code });
        switch (eventType) {
            case JournifyEventType.IDENTIFY:
                if (this.testingMode) {
                    return this.logger.log("Will call window.ttq.identify with the following params in order:", Object.assign(Object.assign({}, event), { event_id: eventId }));
                }
                window.ttq.identify(Object.assign(Object.assign({}, event), { event_id: eventId }));
                break;
            case JournifyEventType.PAGE:
                if (args.event === "Pageview") {
                    if (this.testingMode) {
                        return this.logger.log("Will call window.ttq.page with the following params in order:", Object.assign(Object.assign({}, event), { event_id: eventId }));
                    }
                    window.ttq.page(Object.assign(Object.assign({}, event), { event_id: eventId }));
                }
                else {
                    if (this.testingMode) {
                        return this.logger.log("Will call window.ttq.track with the following params in order:", args.event, event, { event_id: eventId });
                    }
                    window.ttq.track(args.event, event, { event_id: eventId });
                }
                break;
            default:
                if (this.testingMode) {
                    return this.logger.log("Will call window.ttq.track with the following params in order:", args.event, event, { event_id: eventId });
                }
                window.ttq.track(args.event, event, { event_id: eventId });
                break;
        }
    }
    trackPixelEvent(ctx) {
        const event = ctx.getEvent();
        const eventMapping = this.eventMapper.getEventMapping(event);
        if (!eventMapping || !matchFilters(event, eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.filters)) {
            return ctx;
        }
        const mappedProperties = this.fieldsMapper.mapEvent(event);
        const eventId = mappedProperties.event_id;
        delete mappedProperties.event_id;
        const eventName = (eventMapping === null || eventMapping === void 0 ? void 0 : eventMapping.pixelEventName) || event.event;
        this.callPixelHelper(event.type, Object.assign(Object.assign({}, mappedProperties), { event: eventName }), eventId);
        return ctx;
    }
    /* eslint-disable */
    loadTiktokPixel() {
        const w = this.browser.window();
        const d = this.browser.document();
        const t = "ttq";
        w.TiktokAnalyticsObject = t;
        var ttq = (w[t] = w[t] || []);
        (ttq.methods = [
            "page",
            "track",
            "identify",
            "instances",
            "debug",
            "on",
            "off",
            "once",
            "ready",
            "alias",
            "group",
            "enableCookie",
            "disableCookie",
        ]),
            (ttq.setAndDefer = function (t, e) {
                t[e] = function () {
                    t.push([e].concat(Array.prototype.slice.call(arguments, 0)));
                };
            });
        for (var i = 0; i < ttq.methods.length; i++)
            ttq.setAndDefer(ttq, ttq.methods[i]);
        (ttq.instance = function (t) {
            for (var e = ttq._i[t] || [], n = 0; n < ttq.methods.length; n++)
                ttq.setAndDefer(e, ttq.methods[n]);
            return e;
        }),
            (ttq.load = function (e, n) {
                var i = TIKTOK_SCRIPT_URL;
                (ttq._i = ttq._i || {}),
                    (ttq._i[e] = []),
                    (ttq._i[e]._u = i),
                    (ttq._t = ttq._t || {}),
                    (ttq._t[e] = +new Date()),
                    (ttq._o = ttq._o || {}),
                    (ttq._partner = ttq._partner || "Journify"),
                    (ttq._o[e] = n || {});
                var o = d.createElement("script");
                (o.type = "text/javascript"),
                    (o.async = !0),
                    (o.src = i + "?sdkid=" + e + "&lib=" + t);
                var a = d.getElementsByTagName("script")[0];
                a.parentNode.insertBefore(o, a);
            });
        ttq.load(this.settings.pixel_code);
    }
}
//# sourceMappingURL=tiktokPixel.js.map